/*
Example-C Using RMS Block I/O To Copy Files
Last Technical Review: 23-JUN-1989                            Size: 175 lines

Copyright (c) Digital Equipment Corporation, 1991, All Rights Reserved. 
Unpublished Rights Reserved Under The Copyright Laws Of The United States.
               
Any party granted access to this information pursuant to a duly executed
Digital Service Agreement may, under the terms of such agreement, use, copy,
download, store, or transmit such information internally for the exclusive
purpose of maintaining and operating the products covered by such agreement.  
No other use, copying, storage or transmission is permitted except with
Digital's prior written consent.

LAYERED PRODUCT: VAX C, 3.0                    OP/SYS: VMS, V5.n

LAST TECHNICAL REVIEW: 23-JUN-1989

SOURCE: Customer Support Center/Colorado Springs



OVERVIEW:

This program demonstrates how to use RMS in block mode to copy a file
from program control.  It is similar to performing the DCL COPY command.


*** CAUTION ***

This sample program has been tested using VAX C version 3.0 on VMS 5.1-1.
However, we cannot guarantee it's effectiveness because of the
possibility of error in transmitting or implementing it.  It is meant
to be used as a template for writing your own program, and it may
require modification for use on your system.


PROGRAM NOTES:

This program is an example of how one might use RMS in block mode to
imitate the functionality of the DCL COPY command.  There is no
system service to do this directly.  NOTE that the destination file
created by this program is not necessarily identical to the source
file.  The bytes in the last block of the file, after the end-of-file
byte, may not be the same.  However, since RMS doesn't look at
these bytes anyway, the destination file is FUNCTIONALLY identical
to the source file, i.e. the file header information and the use of
the file will be the same.

This program makes use of pointers and dynamically allocated memory
for the RMS control structures that will be used.  It must be run
using a foreign command so that filenames can be passed to it on the
command line.  The first file specified is copied to the second file
specified, which could be the same name.  The RMS routines and data
structures are described in the VMS documentation:

     "VMS Record Management Services Manual", (April 1988),
     Version 5.n

     "VAX Record Management Services Reference Manual", (April 1986),
     Version 4.n

To run this program, extract the following into a command file,
execute the command procedure and then execute these 2 commands
at DCL level:

$ CPY :== $diskname:[directory]copy_rms.exe  ! define a foreign command
$ CPY file1 file2                            ! invoke the program

!$ Command procedure to create the program:
$ CREATE COPY_RMS.C  */
/**********************************************************/
/* Program:  COPY_RMS.C                                   */
/* Purpose:  Copies a file of any type to another file    */
/*           with the same attributes using RMS block IO. */
/**********************************************************/
#include stdio                                        /* Do needed includes  */
#include rms
#define TRUE  1                                       /* Do needed defines   */
#define FALSE 0
#define BUFSIZ 2048

main (argc,argv)                                      /* Command line args   */
 int argc;                                            /*   will be used      */
 char *argv[];
  {
    struct FAB *infab, *outfab;                       /* Ptrs for RMS strucs */
    struct RAB *inrab, *outrab;
    struct XABFHC *infhc, *outfhc;
    int status;                                       /* For status checking */
    char buffer[BUFSIZ];                              /* Buffer for data     */

    if (argc != 3)                                    /* We must have 2 args */
      {
        printf ("Usage:  CPY infile outfile\n");
        exit();
      }

    infab = malloc (sizeof (struct FAB));             /* Allocate memory for */
    inrab = malloc (sizeof (struct RAB));             /*  input RMS structs  */
    infhc = malloc (sizeof (struct XABFHC));

    *infab = cc$rms_fab;                              /* Init input fab      */
    infab->fab$l_fna = argv[1];
    infab->fab$b_fns = strlen (argv[1]);
    infab->fab$b_fac = FAB$M_GET | FAB$M_BIO;         /* Block read only     */
    infab->fab$l_xab = infhc;                         /* We want header info */

    *inrab = cc$rms_rab;                              /* Init input rab      */
    inrab->rab$l_fab = infab;
    inrab->rab$l_ubf = buffer;
    inrab->rab$w_usz = sizeof buffer;

    *infhc = cc$rms_xabfhc;                           /* Init input xabfhc   */

    status = sys$open (infab);                        /* Open input file     */
    if (!(status & 1)) lib$stop(status);
    status = sys$connect (inrab);                     /* Connect rec stream  */
    if (!(status & 1)) lib$stop(status);

    outfab = malloc (sizeof (struct FAB));            /* Allocate memory for */
    outrab = malloc (sizeof (struct RAB));            /*  output RMS structs */
    outfhc = malloc (sizeof (struct XABFHC));

    *outfab = cc$rms_fab;                             /* Init output fab     */
    outfab->fab$b_fac = FAB$M_BIO | FAB$M_PUT;        /* Block write only    */
    outfab->fab$l_fna = argv[2];
    outfab->fab$b_fns = strlen (argv[2]);
    outfab->fab$l_fop = infab->fab$l_fop;             /* Use same values as  */
    outfab->fab$l_alq = infab->fab$l_alq;             /*  the input file had */
    outfab->fab$w_mrs = infab->fab$w_mrs;
    outfab->fab$b_org = infab->fab$b_org;
    outfab->fab$b_rat = infab->fab$b_rat;
    outfab->fab$b_rfm = infab->fab$b_rfm;
    outfab->fab$l_xab = outfhc;

    *outrab = cc$rms_rab;                             /* Init output rab     */
    outrab->rab$l_fab = outfab;
    outrab->rab$b_rac = RAB$C_SEQ;                    /* Sequential access   */
    outrab->rab$l_rbf = buffer;                       /* Buffer to write     */

    *outfhc = cc$rms_xabfhc;                          /* Init output xabfhc  */
    outfhc->xab$w_lrl = infhc->xab$w_lrl;             /* Use longest rec len */

    status = sys$create (outfab);                     /* Create output file  */
    if (!(status & 1)) lib$stop(status);
    status = sys$connect (outrab);                    /* Connect rec stream  */
    if (!(status & 1)) lib$stop(status);

    while ((status = sys$read (inrab)) != RMS$_EOF)   /* Read until EOF      */
      {
        outrab->rab$w_rsz = inrab->rab$w_rsz;         /* Match # bytes read  */
        status = sys$write (outrab);                  /* Write buffer        */
        if (!(status & 1)) lib$stop (status);
      }

    status = sys$close (infab);                       /* Close the files     */
    if (!(status & 1)) lib$stop (status);
    status = sys$close (outfab);
    if (!(status & 1)) lib$stop (status);
  } /*
$ CC copy_rms                                 ! compile the program
$ LINK copy_rms,sys$input/opt                 ! link with shareable VAXCRTL
SYS$SHARE:VAXCRTL.EXE/SHARE
$ DELETE copy_rms.obj;
$ EXIT


HOW TO COPY THIS ARTICLE:

If you are reading this article on the Digital Software Information
Network and would like a copy of it on your system, please refer to
information about how to copy articles available in the Digital
Information Network database.
*/
